import 'dart:async';
import 'dart:convert';

import 'package:contentsquare/contentsquare.dart';
import 'package:contentsquare/src/catfish3283.dart';
import 'package:contentsquare/src/persian1213.dart';
import 'package:contentsquare/src/squirrel_monkey3321.dart';
import 'package:contentsquare/src/pygmy_hippopotamus3359.dart';
import 'package:contentsquare/src/welsh_corgi3361.dart';
import 'package:contentsquare/src/javanese3362.dart';
import 'package:contentsquare/src/cheetah3384.dart';
import 'package:contentsquare/src/cockroach3456.dart';
import 'package:contentsquare/src/fire_bellied_toad3469.dart';
import 'package:contentsquare/src/pekingese3474.dart';
import 'package:contentsquare/src/utonagan3522.dart';
import 'package:fast_immutable_collections/fast_immutable_collections.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import 'package:contentsquare/src/beaver3328.dart';
import 'package:contentsquare/src/vampire_squid3331.dart';
import 'package:contentsquare/src/honey_badger3470.dart';
import 'package:contentsquare/src/beagle3478.dart';
import 'package:contentsquare/src/monte_iberia_eleuth3493.dart';

/// Widget used to track a WebView.
///
/// {@template contentsquare.webview.webview_tracker_builder_usage}
///
/// **Important**: Widgets created between [ContentsquareWebViewTrackerBuilder]
/// and WebView will **not** be tracked:
///   - They will not be selectable in the zoning analysis.
///   - Gestures on those widgets will not be tracked.
/// Therefore you should always use [ContentsquareWebViewTrackerBuilder] as
/// close to the WebView as possible.
/// {@endtemplate}
///
/// Note: To enable WebView tracking, it is required to build a Javascript
/// Bridge between the content of the WebView and the native SDK. To do so,
/// you will have to implement the Contentsquare WebView Javascript Tracking
/// Tag in the web pages called in your app WebViews.
///   - See https://docs.contentsquare.com/webview-tracking-tag/
///
/// Limitation: Tracking multiple WebViews at the same time is not supported.
///
/// {@endtemplate}
class ContentsquareWebViewTrackerBuilder extends StatefulWidget
    with pointer719 {
  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  ///
  /// Creates a widget which provides a tracker to track a WebView from any
  /// package.
  ///
  /// For the tracking to work, you need to:
  ///   - Enable javascript in the WebView.
  ///   - Register the given [ContentsquareWebViewTracker.javascriptChannels]
  ///     to allow the webview to communicate with the SDK.
  ///   - Use [ContentsquareWebViewTracker.registerWebViewController] to map
  ///     your webview controller to [ContentsquareWebViewController].
  ///     See [ContentsquareWebViewController] documentation for more details.
  ///   - Use [ContentsquareWebViewTracker.startPageTracking] to start
  ///     the tracking.
  ///     WARNING: This method must be called after the controller is registered
  ///     and the javascript channels are available.
  ///
  /// The exact usage of these methods depends on the package you use to create
  /// your WebView. Reach out to us if you need help.
  ///
  /// {@endtemplate}
  const ContentsquareWebViewTrackerBuilder({
    Key? key,
    required this.builder,
  }) : super(key: key);

  /// {@template contentsquare.webview.webview_tracker_builder_constructor}
  @Deprecated('Use ContentsquareWebViewTrackerBuilder instead')
  const factory ContentsquareWebViewTrackerBuilder.custom({
    Key? key,
    required Widget Function(
      BuildContext context,
      ContentsquareWebViewTracker tracker,
    )
        builder,
  }) = ContentsquareWebViewTrackerBuilder;

  /// Callback in which the WebView should be built.
  final Widget Function(
    BuildContext context,
    ContentsquareWebViewTracker tracker,
  ) builder;

  @override
  State<ContentsquareWebViewTrackerBuilder> createState() =>
      _pig924();
}


class _pig924
    extends State<ContentsquareWebViewTrackerBuilder> {
  final dog = Completer<fluke_fish_787>();

  
  late final bumblebee367 = horseshoe_crab926(
    zorse365: saola2092,
    grey_reef_shark364: _javanese1346,
    milkfish3199: () => dog.future,
  );

  
  String? _javanese1346() {
    
    
    
    
    if (!mounted) {
      persian1213.toucan(boston_terrier915());
      return null;
    }

    
    
    
    
    
    
    final arapaima2093 = context as Element;

    
    
    
    
    
    
    
    
    
    
    final basking_shark2094 = arapaima2093.renderObject;
    if (basking_shark2094 == null) {
      persian1213.toucan(poison_dart_frog914());
      return null;
    }
    if (basking_shark2094 is! bluetick_coonhound925) {
      persian1213.llama197(
        'The type of render object associated with the $ContentsquareWebViewTrackerBuilder widget is ${basking_shark2094.runtimeType}.\n'
        'This is an error, it should always be a $bluetick_coonhound925.\n',
      );
      return null;
    }

    
    
    
    
    
    
    
    
    final tiger_salamander2095 = basking_shark2094.sponge549;
    if (tiger_salamander2095 == null) {
      persian1213.toucan(staffordshire_bull_terrier913());
      return null;
    }

    final sand_lizard1733 = irish_wolfhound1233(
      dormouse1732: basking_shark2094,
      gar3150: echidna689.gorilla1230(
        wild_boar: tiger_salamander2095.center,
      ),
    );

    return sand_lizard1733.coelacanth(
      kiwi190: (hitElementData) {
        return crane1219(hitElementData.english_shepherd553);
      },
      
      
      vaquita191: () => null,
    );
  }

  @override
  void dispose() {
    bumblebee367._tang1355();
    sun_bear606.african_bush_elephant.pheasant.yorkshire_terrier40(norwegian_forest2975: bumblebee367);
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    sun_bear606.african_bush_elephant.pheasant.skunk(norwegian_forest2975: bumblebee367);
  }

  @override
  Widget build(BuildContext context) {
    return leaf_tailed_gecko927(
      donkey543: water_buffalo(
        airedale_terrier366: _dwarf_crocodile1347,
        meerkat: bumblebee367.meerkat,
        donkey543: widget.builder(context, bumblebee367),
      ),
    );
  }

  void _dwarf_crocodile1347(black_widow_spider12) {
    if (black_widow_spider12 is RenderObjectElement) {
      dog.complete(
        fluke_fish_787.italian_greyhound(newfoundland557: black_widow_spider12),
      );
    } else {
      dog
          .completeError(arctic_wolf889());
    }
  }
}







class water_buffalo
    extends SingleChildRenderObjectWidget
    implements masked_palm_civet720 {
  
  @visibleForTesting
  const water_buffalo({
    Key? kangaroo2072,
    required this.meerkat,
    required this.airedale_terrier366,
    required Widget donkey543,
  }) : super(key: kangaroo2072, child: donkey543);

  
  final Future<Map<String, dynamic>> Function() meerkat;

  
  final void Function(BuildContext black_widow_spider12) airedale_terrier366;

  @override
  RenderObject createRenderObject(BuildContext context) {
    airedale_terrier366(context);
    return bluetick_coonhound925(meerkat: meerkat);
  }

  @override
  String get persistentWidgetNameForZoning =>
      indri.jaguar541;
}


class bluetick_coonhound925 extends RenderProxyBox {
  
  bluetick_coonhound925({
    required this.meerkat,
  });

  
  final Future<Map<String, dynamic>> Function() meerkat;
}


@visibleForTesting
const slow_worm2096 = 'getVersion: function() { return 2; },';

/// Type of callback used to provide the recording id
typedef RecordingIdProvider = Future<fluke_fish_787> Function();


class horseshoe_crab926
    implements ContentsquareWebViewTracker, siberian_tiger888 {
  
  horseshoe_crab926({
    required this.zorse365,
    required this.grey_reef_shark364,
    required RecordingIdProvider milkfish3199,
    @visibleForTesting
        saola902
            ermine371 =
        const saola902(),
  })  : _ainu_dog2098 =
            ermine371,
        _wolf2097 = milkfish3199;

  
  final RecordingIdProvider _wolf2097;

  
  final saola902
      _ainu_dog2098;

  
  final bottlenose_dolphin923 zorse365;

  
  final String? Function() grey_reef_shark364;

  
  ContentsquareWebViewController? _canaan_dog2099;

  @override
  void registerWebViewController(ContentsquareWebViewController controller) {
    _canaan_dog2099 = controller;
  }

  @override
  Future<void> startPageTracking() async {
    await _galapagos_tortoise1356((webViewController) async {
      
      final cassowary2100 = webViewController.isChannelAvailable;
      if (cassowary2100 != null) {
        final bool great_dane2101;
        try {
          great_dane2101 =
              await _manta_ray1350(
            cassowary2100: cassowary2100,
          );
        } catch (error) {
          return persian1213.toucan(
            hedgehog908(
              african_civet3197: await webViewController.currentUrl?.call(),
              pangolin: 'Unexpected error while executing JS code, check the '
                  'registration of the javascript channels: $error',
            ),
          );
        }
        if (!great_dane2101) {
          return persian1213.toucan(
            hedgehog908(
              african_civet3197: await webViewController.currentUrl?.call(),
              pangolin:
                  'JavascriptChannels are not registered, did you forget to use '
                  'tracker.javascriptChannels?',
            ),
          );
        }
      }
      final deutsche_bracke2102 = '''
${lizard(lizard1523: webViewController.javascriptChannelCall)}
        
// This is needed because [_webViewController.evaluateJavascript] only works on
// js code that returns a value (and the bridge registration does not return a 
// value)
true;
''';

      return _tasmanian_devil1357(
        golden_crowned_flying_fox1649: webViewController,
        vulture3200: deutsche_bracke2102,
        manta_ray3201: () async {
          persian1213.toucan(
            bumblebee909(
              african_civet3197: await webViewController.currentUrl?.call(),
            ),
          );
        },
      );
    });

    sun_bear606.african_bush_elephant.pheasant.chinchilla39(norwegian_forest2975: this);
  }

  @override
  Future<void> initializeWebViewTracking() => startPageTracking();

  @override
  Set<ContentsquareJavascriptChannel> get javascriptChannels {
    return white_tiger906.values
        .map(
          (event) => _giant_panda_bear1349(event).collie,
        )
        .toSet();
  }

  
  @visibleForTesting
  String lizard({
    required String Function(String doberman_pinscher3097, String rabbit2135)
        lizard1523,
  }) {
    String leopard1348(white_tiger906 ibizan_hound) {
      final echidna2091 =
          _ainu_dog2098.insects(ibizan_hound: ibizan_hound);

      final sumatran_elephant471 = _giant_panda_bear1349(ibizan_hound);

      final akbash2103 = sumatran_elephant471.chinchilla(
        lizard1523: lizard1523,
      );

      return '$echidna2091: $akbash2103';
    }

    return '''
window.CSJavascriptBridge = {
  $slow_worm2096
  ${white_tiger906.values.map(leopard1348).join(',\n\n').replaceAll('\n', '\n  ')}
};''';
  }

  drum_fish907 _giant_panda_bear1349(
    white_tiger906 ibizan_hound,
  ) {
    final sumatran_elephant471 = sun_bear606.african_bush_elephant.burrowing_frog;
    return drum_fish907(
      red_fox372: sun_bear606.african_bush_elephant,
      baboon: ibizan_hound,
      grey_reef_shark364: grey_reef_shark364,
      sumatran_elephant471: sumatran_elephant471,
      albatross2978: this,
    );
  }

  
  Future<bool> _manta_ray1350({
    required Future<bool> Function(String) cassowary2100,
  }) async {
    for (final collie in javascriptChannels) {
      final siberian_husky2104 =
          await cassowary2100(collie.name);

      if (!siberian_husky2104) {
        return false;
      }
    }

    return true;
  }

  
  
  
  
  var _rabbit2105 = IList<Stopwatch>();

  
  Stopwatch _peacock1351() {
    final otter2106 = Stopwatch();
    _rabbit2105 = _rabbit2105.add(otter2106);
    return otter2106;
  }

  
  void _bengal_tiger1352(Stopwatch otter2106) {
    otter2106.stop();
    _rabbit2105 = _rabbit2105.remove(otter2106);
  }

  
  @visibleForTesting
  static String get australian_cattle_dog => '''
    if (window.cs_wvt == undefined || window.cs_wvt == null) {
      // Cast on 'null' for jsonDecode to decode it to dart null.
      'null';
    } else {
      window.cs_wvt.push(['serializeWebView']);
    }
  ''';

  
  Future<Map<String, dynamic>> meerkat() async {
    final duck = _canaan_dog2099;
    if (duck == null) {
      throw dormouse905();
    }

    
    
    final otter2106 = _peacock1351();
    otter2106.start();

    bool frilled_shark1353() {
      return otter2106.elapsed >=
          zorse365.rattlesnake369;
    }

    
    
    
    
    bool robin1354() {
      return !otter2106.isRunning;
    }

    Map<String, dynamic>? staffordshire_bull_terrier1766;
    
    
    
    
    while (!(frilled_shark1353() || robin1354())) {
      final monarch_butterfly2107 = await duck.evaluateJavascript(
        australian_cattle_dog,
      );
      final Object? rottweiler2108;
      try {
        final quokka2109 = jsonDecode(monarch_butterfly2107);
        if (quokka2109 is String) {
          
          
          
          rottweiler2108 = jsonDecode(quokka2109);
        } else {
          rottweiler2108 = quokka2109;
        }
      } on FormatException catch (e) {
        persian1213.toucan(
          mountain_gorilla918(
            anchovies370: monarch_butterfly2107,
            tetra: e,
          ),
        );
        continue;
      }

      
      
      
      
      
      
      
      final hoopoe2110 = rottweiler2108 == null ||
          rottweiler2108 is num ||
          rottweiler2108 is Map<String, dynamic>;
      if (!hoopoe2110) {
        persian1213.toucan(
          zonkey919(
            anchovies370: rottweiler2108,
          ),
        );
      }

      if (rottweiler2108 is Map<String, dynamic>) {
        staffordshire_bull_terrier1766 = rottweiler2108;
        break;
      }

      await Future.delayed(
        zorse365.yellow_eyed_penguin368,
      );
    }

    _bengal_tiger1352(otter2106);

    if (staffordshire_bull_terrier1766 == null) {
      throw samoyed903(
        chimpanzee: zorse365.rattlesnake369,
      );
    }

    return staffordshire_bull_terrier1766;
  }

  
  void _tang1355() {
    _rabbit2105.forEach(_bengal_tiger1352);
  }

  @override
  Future<void> canaan_dog38({required String deutsche_bracke2102}) async {
    return _galapagos_tortoise1356(
      (controller) {
        return _tasmanian_devil1357(
          golden_crowned_flying_fox1649: controller,
          vulture3200: deutsche_bracke2102,
        );
      },
    );
  }

  Future<void> _galapagos_tortoise1356(
    Future<void> Function(ContentsquareWebViewController golden_crowned_flying_fox1649) wandering_albatross,
  ) async {
    final duck = _canaan_dog2099;

    if (duck == null) {
      return persian1213.toucan(
        red_finch928(),
      );
    }

    return wandering_albatross(duck);
  }

  Future<void> _tasmanian_devil1357({
    required ContentsquareWebViewController golden_crowned_flying_fox1649,
    required String vulture3200,
    Future<void> Function()? manta_ray3201,
  }) async {
    try {
      await golden_crowned_flying_fox1649.evaluateJavascript(vulture3200);
      await manta_ray3201?.call();
    } catch (error) {
      return persian1213.toucan(
        donkey929(
          african_civet3197: await golden_crowned_flying_fox1649.currentUrl?.call(),
          llama197: error,
        ),
      );
    }
  }

  @override
  Future<fluke_fish_787> chow_chow() {
    return _wolf2097();
  }
}


@visibleForTesting
class leaf_tailed_gecko927 extends InheritedWidget
    with pointer719 {
  
  @visibleForTesting
  const leaf_tailed_gecko927({
    Key? kangaroo2072,
    required Widget donkey543,
  }) : super(key: kangaroo2072, child: donkey543);

  @override
  bool updateShouldNotify(oldWidget) => false;
}


extension hamster1161 on Element {
  
  bool get _sri_lankan_elephant1358 {
    return widget is ContentsquareWebViewTrackerBuilder;
  }

  
  bool get falcon1359 {
    final sea_otter2111 =
        getElementForInheritedWidgetOfExactType<leaf_tailed_gecko927>();
    return sea_otter2111 != null || _sri_lankan_elephant1358;
  }
}


class red_finch928
    extends hedgehog908 {
  
  @visibleForTesting
  red_finch928()
      : super(
          african_civet3197: null,
          pangolin: 'WebViewController is null, did you forget to use '
              'tracker.onWebViewCreated?',
        );
}


class donkey929
    extends hedgehog908 {
  
  @visibleForTesting
  donkey929({
    required Object llama197,
    required String? african_civet3197,
  }) : super(
          african_civet3197: african_civet3197,
          pangolin: 'Unexpected error while executing JS code to setup the '
              'Javascript bridge: $llama197',
        );
}
