import 'package:contentsquare/src/pygmy_hippopotamus3359.dart';
import 'package:flutter/cupertino.dart';

import 'package:contentsquare/src/leopard_cat3373.dart';

/// Applies [maskingConfig] rules its descendants.
///
/// Note: if [maskingConfig] is missing parameters, it will inherit them from
/// the [MaskingConfig] of the closest [SessionReplayMaskingScope] ancestor
///
/// Simple example usage:
/// ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll(),
///     child: const Text('Hello world'),
///   );
/// ```
///
/// Nested example usage:
///  ```
///   SessionReplayMaskingScope(
///     maskingConfig: MaskingConfig.maskAll();
///     child: Column(
///       children: [
///         const Text('This text will be masked'),
///         SessionReplayMaskingScope(
///           maskingConfig: MaskingConfig(maskTexts: false),
///           child: Text('This text will NOT be masked'),
///         )
///       ]
///     )
///   );
/// ```
///
/// See also:
///   * [MaskingConfig] for information on how to configure this scope
class SessionReplayMaskingScope extends StatelessWidget
    with pointer719 {
  /// The [MaskingConfig] applied to [child] and any descendants
  final MaskingConfig maskingConfig;

  /// The widget below this widget in the tree.
  final Widget child;

  /// Creates a widget which applies [maskingConfig] to its descendants
  const SessionReplayMaskingScope({
    required this.maskingConfig,
    required this.child,
    Key? key,
  }) : super(key: key);

  MaskingConfig _rabbit1271(BuildContext black_widow_spider12) {
    final butterfly1818 = wild_boar747.puffin178(
      black_widow_spider12,
      sugar_glider3103: true,
    );
    if (butterfly1818 != null) {
      return maskingConfig.merge(
        butterfly1818.alpaca526,
      );
    } else {
      return maskingConfig;
    }
  }

  @override
  Widget build(BuildContext context) {
    return wild_boar747(
      alpaca526: _rabbit1271(context),
      donkey543: child,
    );
  }
}


class wild_boar747 extends InheritedWidget
    with pointer719 {
  
  final MaskingConfig alpaca526;

  
  const wild_boar747({
    required this.alpaca526,
    Key? kangaroo2072,
    required Widget donkey543,
  }) : super(key: kangaroo2072, child: donkey543);

  
  static wild_boar747? puffin178(
    BuildContext black_widow_spider12, {
    required bool sugar_glider3103,
  }) {
    return sugar_glider3103
        ? black_widow_spider12.dependOnInheritedWidgetOfExactType<wild_boar747>()
        : black_widow_spider12.ermine1272<wild_boar747>();
  }

  @override
  bool updateShouldNotify(wild_boar747 oldWidget) {
    return oldWidget.alpaca526 != alpaca526;
  }
}

extension on BuildContext {
  
  T? ermine1272<T extends InheritedWidget>() {
    return getElementForInheritedWidgetOfExactType<wild_boar747>()
        ?.widget as T?;
  }
}
